<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DTO\Request;
use HCM\BTL\Services\FasesTemplateService;

class CandidatosAprovadosController extends Crud {

    protected $dataSourceName = 'des_participafase'; 
    protected $dataSourceName2 = 'des_participafase_verifica';

    protected $parameterBagColumns = array('NRORG', 'NRORDEM', 'NRFASE', 'NRPROCESSO', 'NRTEMPLATE');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, FasesTemplateService $fasesTemplateService){
        $this->parameterBag = $parameterBag;
        $this->fasesTemplateService = $fasesTemplateService;
        parent::__construct($dataSourceManager);
    }

    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName2);        
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);  
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function trocarFaseCandidato (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $params = $request->getDataSet()->getRows()[0];
            $nrfase = intval($this->fasesTemplateService->getNextNrfase($params['NRORG'], $params['NRTEMPLATE'], $params['NRORDEM']));
            if($nrfase === null){
                return false;
            } else {
                $aux = $this->dataSourceName;
                $this->dataSourceName = $this->dataSourceName;
                foreach($request->getDataSet()->getRows() as &$row) {
                    foreach ($row['CANDIDATOS'] as $rows) {
                        if(array_key_exists('NRDURACAO', $rows)){
                            $rows['NRDURACAO'] = DateUtil::getHoraFloatDeString($rows['NRDURACAO']);
                        }
                        $rows['__is_new'] = true;

                        $rows['NRPARCTICIPAFASE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_PARTICIPAFASE', 1, 12, true);

                        $rows['NRFASE'] = $nrfase;

                        $rows['DTULTATU'] = date('d/m/Y');

                        $rows['NRORGULTATU'] = $row['NRORG'];

                        $rows['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];

                        $rows['NRPERCENTUAL'] = str_replace("%", "", $rows['NRPERCENTUAL']);

                        $rows['NRDURACAO'] = null;

                        $rows['NRPESO'] = null;

                        $rows['DTINICIO'] = null;

                        $rows['DTFIM'] = null;

                        $rows['NRAVALIACAO'] = null;

                        $rows['DSLOCALFASE'] = null;

                        $rows['IDOBRIGATORIO'] = null;

                        $rows['DTINICIO'] = null;

                        $rows['DTFIM'] = null;

                        $rows['NRPONTUACAO'] = null;

                        $rows['NRAVALIACAO'] = null;

                        $rows['IDSTATUS'] = null;

                        $rows['CDTIPORESU'] = null;

                        $rows['NRPERCENTUAL'] = null;

                        $requestSave = new DTO\Request\DataSet ((new \Zeedhi\Framework\DataSource\DataSet('des_participafase', array($rows))), $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                        parent::save($requestSave, $response);
                    }
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}