<?php


namespace HCM\Geral\Entity;

use HCM\Util\Repositories;
use HCM\Util\TkEntity;

class GpepessoaEntity extends TkEntity
{

    private $_nrOrg;
    private $_mesCompetencia;
    private $_historicos;
    public $historico;

    public function __construct($entityManager, $hcmParameters)
    {

        parent::__construct(null,$entityManager);
       
        $this->hcmParameters = $hcmParameters;

        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_mesCompetencia = $hcmParameters->getDtCompetencia();
    }

    public function retornaPessoa($nrOrg, $nrPessoa)
    {
        $pessoa = Gpepessoa::retornaPessoa($nrOrg, $nrPessoa);
        return $pessoa;
    }

    public function retornaPessoaAtual($nrPessoa)
    {
        return $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaAtual(
                $this->_nrOrg, $nrPessoa, $this->_mesCompetencia
            );
    }
    
    public function retornaPessoaPorParceiro($nr_parcnegocio)
    {
        return $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaPorParceiro(
                $this->_nrOrg, $nr_parcnegocio, $this->_mesCompetencia
            );
    }

    public function incluiPessoa($nrparcnegocio, $nrorg = null)
    {

        if ($nrorg != null) {
            $this->_nrOrg = $nrorg;
        }

        try {
            $pessoa = new self();

            $pessoa->setNrorg($this->_nrOrg);
            $pessoa->setNrparcnegocio($nrparcnegocio);

            $pessoa->persist();

            $nrPessoa = $pessoa->getNrpessoa();
            unset($pessoa);
            return $nrPessoa;
        } catch (Exception $e) {
            throw new Exception(
                ' Exception ' . $e->getLine() . ' : <br />' . $e->getTrace() . ' - ' . $e->getMessage()
            );
        }
    }

    public function inserePessoa($nrorg, $nrparcnegocio)
    {
        $this->setNrorg($nrorg);
        $this->setNrparcnegocio($nrparcnegocio);
    }

    public function getHistoricos()
    {
        if (!$this->_historicos) {
            $this->_historicos = self::getEntityByWhere(
                'NRORG = ? AND NRPESSOA = ?',
                array($this->getNrorg(), $this->getNrpessoa())
            );
        }
        return $this->_historicos;
    }

    public function beforeInsertPessoa()
    {

        $db = Connection::getInstance()->getConn();
        $query = "SELECT MAX(NRPESSOA) FROM GPE_PESSOA";
        $max = (int)$db->execute($query)->fetchColumn(0);

        $this->setNrpessoa($max + 1);

    }

    public function getPessoa($nrorg, $nrpessoa)
    {
        $params = array(
            'NRORG' => $nrorg,
            'NRPESSOA' => $nrpessoa
        );
        $pessoa = self::getEntityByWhere('NRORG = :NRORG AND NRPESSOA = :NRPESSOA', $params);
        return $pessoa;
    }

    public function getHistoricoCompetencia($mescompetencia)
    {
        $gpepessoah = Gpepessoah::getUltimoHistoricoCompetencia(
            $this->getNrorg(), $this->getNrpessoa(), $mescompetencia
        );

        return $gpepessoah;
    }

    public function getUltimoHistorico()
    {
        if (!$this->historico) {
            $params = array(
                'NRORG' => $this->getNrorg(),
                'NRPESSOA' => $this->getNrpessoa()
            );
            $this->historico = GpepessoahEntity::getEntityByWhere(
                '   NRORG          = :NRORG
                AND NRPESSOA       = :NRPESSOA
                AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                       FROM GPE_PESSOAH
                                      WHERE NRORG    = :NRORG
                                        AND NRPESSOA = :NRPESSOA)',
                $params
            );
        }
        return $this->historico;
    }
    
    public function verificaPessoaDeficiente($nrorg, $nrpessoa, $dtcompetencia) {
        try {
            $pessoa = new GpepessoahEntity($this->entityManager, $this->hcmParameters);
            $pessoah = $pessoa->retornaHistoricoAtualPorPessoa($nrorg, $nrpessoa, $dtcompetencia);
            
            if ($pessoah && $pessoah->getNrcondfispes() && ($pessoah->getNrcondfispes() !== 1)) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }    

}

