<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\ExportacaoExcelService;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Zeedhi\Framework\HTTP\Kernel;

class EsocialBxService {
    
    private $arrRegistros = array();

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, Kernel $httpKernel, ExportacaoExcelService $exportacaoExcelService) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->httpKernel = $httpKernel;
        $this->exportacaoExcelService = $exportacaoExcelService;
    }

    public function cadastroAutomaticoBxEsocial($nrestruturam, $nrtpambiente, $arrNrevento, $arrCdperiodoapuracao) {
        try {
            $result = true;
            $msg = array();
            
            $this->entityManager->beginTransaction();
            foreach ($arrNrevento as $nrevento) {
                $esoEvento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $nrevento));
                
                if (is_object($esoEvento)) {
                    if ($esoEvento->getIdgrupo() == 'I') {
                        if (empty($arrCdperiodoapuracao)) {
                            $this->novoEsoBxEsocial($this->hcmParameters->getNrorgTrabalho(), $nrestruturam, $nrtpambiente, $nrevento, null, null, null, null);
                        } else {
                            foreach ($arrCdperiodoapuracao as $cdperiodoapuracao) {
                                $this->novoEsoBxEsocial($this->hcmParameters->getNrorgTrabalho(), $nrestruturam, $nrtpambiente, $nrevento, null, null, null, $cdperiodoapuracao);
                            }
                        }
                    } else {
                        $this->novoEsoBxEsocial($this->hcmParameters->getNrorgTrabalho(), $nrestruturam, $nrtpambiente, $nrevento, null, null, null, null);
                    }
                } else {
                    $msg[] = "Evento $nrevento não encontrado";
                }
            }
            $this->entityManager->commit();
            
            return array(
                'result' => $result,
                'msg' => $msg,
            );
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function novoEsoBxEsocial($nrorg, $nrestruturam, $nrtpambiente, $nrevento, $nrcpftrabalhador, $dtiniperiodo, $dtfimperiodo, $cdperiodoapuracao) {
        try {
            $esoBxesocial = Repositories::ESO_BXESOCIAL;
            
            $novo = new $esoBxesocial;
            
            $novo->setNrorg($nrorg);
            $novo->setNrestruturam($nrestruturam);
            $novo->setNrtpambiente($nrtpambiente);
            $novo->setNrevento($nrevento);
            $novo->setNrcpftrabalhador($nrcpftrabalhador);
            $novo->setDtiniperiodo($dtiniperiodo);
            $novo->setDtfimperiodo($dtfimperiodo);
            $novo->setCdperiodoapuracao($cdperiodoapuracao);
            $novo->setIdstatus('P');
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            
            $novo->setNrbxesocial(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_BXESOCIAL', 1, 12, true));

            $this->entityManager->persist($novo);
            $this->entityManager->flush();
            
            return $novo;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function gerarTemplatePlanilhaCPF() {
        try{
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            
            $cabecalho = array(
                "CPF",
                "Data Inicial",
                "Data Final"
            );
            
            // Dados fictícios para a planilha
            $dados = array(
                array(
                    "nrcpftrabalhador"  => '11122233344', 
                    "dtiniperiodo"   => '01/01/2017',
                    "dtfimperiodo" => '01/06/2017'
                ),
                array(
                    "nrcpftrabalhador"  => '11223344556', 
                    "dtiniperiodo"   => '01/10/2017',
                    "dtfimperiodo" => '30/10/2017'
                ),
                array(
                    "nrcpftrabalhador"  => '12312312312', 
                    "dtiniperiodo"   => '01/03/2017',
                    "dtfimperiodo" => '31/10/2017'
                )
            );
                
            return $this->exportacaoExcelService->gravarTemplate($dados, $nrorg, $cabecalho, "Planilha de CPF");
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function importarPlanilhaCPFBxEsocial($nrestruturam, $nrtpambiente, $arquivo){
        try{
            $this->entityManager->beginTransaction();
            
            $this->mensagens = [];
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtIniNaoPer = ($nrtpambiente == '1') ? $this->hcmParameters->getEsoIniValidNaoPerProducao() : $this->hcmParameters->getEsoIniValidNaoPerProdRest();
            $salvo = true;
            
            $infos = $this->preparaImportacao($nrorg, $arquivo);
            
            if(is_array($infos)){
                foreach($infos as $key => $info) {
                    
                    $nrcpftrabalhador = $info['NRCPFTRABALHADOR'];
                    
                    $dtiniperiodo = DateUtil::getDataDeString($info['DTINIPERIODO'], DateUtil::FORMATO_BRASILEIRO, true);
                    if(is_null($dtiniperiodo)){
                        $dtiniperiodo = $dtIniNaoPer;
                    } else {
                        $dtiniperiodo = $dtiniperiodo;
                    }
                    
                    $dtfimperiodo = DateUtil::getDataDeString($info['DTFIMPERIODO'], DateUtil::FORMATO_BRASILEIRO, true);
                    if(is_null($dtfimperiodo)){
                        $dtfimperiodo = DateUtil::getDataAtual();
                    } else {
                        $dtfimperiodo = $dtfimperiodo;
                    }
                    
                    $this->novoEsoBxEsocial($nrorg, $nrestruturam, $nrtpambiente, null, $nrcpftrabalhador, $dtiniperiodo, $dtfimperiodo, null);
                }
                if(!empty($this->mensagens)){
                    $salvo = false;
                }
            } else {
                $salvo = false;
                $this->mensagens[] = "Nâo foram importados registros";
            }
            
            $this->entityManager->commit();
            return array('success'  => $salvo,
                         'messages' => $this->mensagens);
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }    
    }
    
    public function preparaImportacao($nrorg, $arquivo) {
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
        
        if (!$file->isFile()) {
            $this->mensagens[] = "Falha ao importar o arquivo.";
        }
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        $this->validaExtensao($caminhoNovo, $arquivo['name']);
        return $this->preparaInformacao($caminhoNovo, $arquivo['name']);
        
    }
    
    private function validaExtensao($caminho, $arquivo) {
        $nome = preg_split("/(\\|\/|\.)/", $caminho);
        $extencao = strtoupper(end($nome));
        switch ($extencao) {
            case('XLS'):{
                return 1;
            }
            case('XLSX'):{
                return 2;
            }
            default:{
                $this->mensagens[] = "Extensão Inválida. (" . $arquivo . ")";
            }
        }
    }
    
    public function preparaInformacao($caminho, $nome) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $file = \PHPExcel_IOFactory::load($caminho);
            $file->setActiveSheetIndex(0);
            
            if($file->getActiveSheet()->getCell('A1')->getValue() != 'CPF'){
                $this->mensagens[] = 'Planilha com dados inválidos! Favor exportar a planilha template.';
                return array();
            }
            
            $eventos = $this->exportacaoExcelService->preparaInfoEsoBxEsocial($file);
            
            if(empty($eventos)){
                $this->mensagens[] = 'Não foi possível importar! Para importação o CPF deve estar informado.';
            }
            return $eventos;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function preparaMensagens($mensagens){
        $arrayRetorno = array();
        foreach($mensagens as $mensagem){
            $arrayRetorno[] = ['msg' => $mensagem];
        }
        return $arrayRetorno;
    }

    /**
     * Retorna o xml de envio, retorno de envio e retorno de consulta dos eventos
     * @param $nrorg
     * @param $xmlFile
     */
    public function getXmlFiles($nrorg, $xmlFile) {
        try {
            
            // SOLUÇÃO NOVO FILESERVER	 
            // Subsitituir a função por:	 
            // $fileServePath = $this->hcmParameters->getFileServerDirectory() . '/files/'. $dsDirXml;	 
            // $file = Util::buscaArquivoFileServer($fileServePath . '.xml', 'xml', false);	 
            // // $file é o conteúdo do arquivo buscado	 
            // return $file;

            $file = $this->hcmParameters->getFileServerDirectory() . DIRECTORY_SEPARATOR . $xmlFile;
            error_reporting(E_ALL ^ E_WARNING);
            if (fopen($file, 'r')) {
                $xml = file_get_contents($file);
            } else {
                throw new \Exception(Messages::FILE_NOT_FOUND);
            }
            error_reporting(E_ALL ^ E_NOTICE);
            return $xml;
        } catch(\Exception $e) {
            throw $e;
        }
    }

}